# Authors: Robert Luke <mail@robertluke.net>
#
# License: BSD (3-clause)

import os

import mne
import numpy as np

import mne_nirs
from mne_nirs.experimental_design import (
    drift_high_pass,
    longest_inter_annotation_interval,
    make_first_level_design_matrix,
)
from mne_nirs.simulation import simulate_nirs_raw


def _load_dataset():
    """Load data and tidy it a bit"""
    fnirs_data_folder = mne.datasets.fnirs_motor.data_path()
    fnirs_raw_dir = os.path.join(fnirs_data_folder, "Participant-1")
    raw_intensity = mne.io.read_raw_nirx(fnirs_raw_dir, verbose=True).load_data()

    raw_intensity.crop(0, raw_intensity.annotations.onset[-1])

    new_des = [des for des in raw_intensity.annotations.description]
    new_des = ["A" if x == "1.0" else x for x in new_des]
    new_des = ["B" if x == "2.0" else x for x in new_des]
    new_des = ["C" if x == "3.0" else x for x in new_des]
    annot = mne.Annotations(
        raw_intensity.annotations.onset, raw_intensity.annotations.duration, new_des
    )
    raw_intensity.set_annotations(annot)

    picks = mne.pick_types(raw_intensity.info, meg=False, fnirs=True)
    dists = mne.preprocessing.nirs.source_detector_distances(
        raw_intensity.info, picks=picks
    )
    raw_intensity.pick(picks[dists > 0.01])

    assert "fnirs_cw_amplitude" in raw_intensity
    assert len(np.unique(raw_intensity.annotations.description)) == 4

    return raw_intensity


def test_create_boxcar():
    raw_intensity = _load_dataset()
    raw_intensity = raw_intensity.pick(picks=[0])  # Keep the test fast
    bc = mne_nirs.experimental_design.create_boxcar(raw_intensity)

    assert bc.shape[0] == raw_intensity._data.shape[1]
    assert bc.shape[1] == len(np.unique(raw_intensity.annotations.description))

    assert np.max(bc) == 1
    assert np.min(bc) == 0

    # The value of the boxcar should be 1 when a trigger fires
    assert (
        bc[int(raw_intensity.annotations.onset[0] * raw_intensity.info["sfreq"]), :][0]
        == 1
    )

    # Only one condition was ever present at a time in this data
    # So boxcar should never overlap across channels
    assert np.max(np.mean(bc, axis=1)) * bc.shape[1] == 1


def test_create_design():
    raw_intensity = _load_dataset()
    raw_intensity.crop(450, 600)  # Keep the test fast
    design_matrix = make_first_level_design_matrix(
        raw_intensity, drift_order=1, drift_model="polynomial"
    )

    assert design_matrix.shape[0] == raw_intensity._data.shape[1]
    # Number of columns is number of conditions plus the drift plus constant
    assert (
        design_matrix.shape[1]
        == len(np.unique(raw_intensity.annotations.description)) + 2
    )


def test_cropped_raw():
    # Ensure timing is correct for cropped signals
    raw = simulate_nirs_raw(
        sfreq=1.0,
        amplitude=1.0,
        sig_dur=300.0,
        stim_dur=1.0,
        isi_min=20.0,
        isi_max=40.0,
    )

    onsets = raw.annotations.onset
    onsets_after_crop = [onsets[idx] for idx in np.where(onsets > 100)]

    raw.crop(tmin=100)
    design_matrix = make_first_level_design_matrix(
        raw, drift_order=0, drift_model="polynomial"
    )

    # 100 corrects for the crop time above
    # 4 is peak time after onset
    new_idx = np.round(onsets_after_crop[0][0]) - 100 + 4
    assert design_matrix["A"][new_idx] > 0.09


def test_high_pass_helpers():
    # Test the helpers give reasonable values
    raw = simulate_nirs_raw(
        sfreq=1.0,
        amplitude=1.0,
        sig_dur=300.0,
        stim_dur=1.0,
        isi_min=20.0,
        isi_max=38.0,
    )
    lisi, names = longest_inter_annotation_interval(raw)
    lisi = lisi[0]
    assert lisi >= 20
    assert lisi <= 40
    assert drift_high_pass(raw) >= 1 / (40 * 2)
    assert drift_high_pass(raw) <= 1 / (20 * 2)



# Authors: Robert Luke <mail@robertluke.net>
#
# License: BSD (3-clause)

import os
from copy import deepcopy

import numpy as np
from mne import EvokedArray, Info, read_source_spaces, stc_near_sensors
from mne.io.constants import FIFF
from mne.utils import get_subjects_dir, verbose


@verbose
def plot_glm_surface_projection(
    inst,
    statsmodel_df,
    picks="hbo",
    value="Coef.",
    background="w",
    figure=None,
    clim="auto",
    mode="weighted",
    colormap="RdBu_r",
    surface="pial",
    hemi="both",
    size=800,
    view=None,
    colorbar=True,
    distance=0.03,
    subjects_dir=None,
    src=None,
    verbose=False,
):
    """
    Project GLM results on to the surface of the brain.

    Note: This function provides a convenient wrapper around low level
    MNE-Python functions. It is convenient if you wish to use a generic head
    model. If you have acquired fMRI images you may wish to use the underlying
    lower level functions.

    Note: This function does not conduct a forward model analysis with photon
    migration etc. It simply projects the values from each channel to the
    local cortical surface. It is useful for visualisation, but users should
    take this in to consideration when drawing conclusions from the
    visualisation.

    Parameters
    ----------
    inst : instance of Raw
        Haemoglobin data.
    statsmodel_df : dataframe
        As produced by produced by `statsmodels_to_results`.
    %(picks_base)s good sEEG, ECoG, and DBS channels.
    value : str
        Column from dataframe to plot.
    background : matplotlib color
        Color of the background of the display window.
    figure : mayavi.core.api.Scene, matplotlib.figure.Figure, list, int, None
        If None, a new figure will be created. If multiple views or a
        split view is requested, this must be a list of the appropriate
        length. If int is provided it will be used to identify the Mayavi
        figure by it's id or create a new figure with the given id. If an
        instance of matplotlib figure, mpl backend is used for plotting.
    %(clim)s
    mode : str
        Can be "sum" to do a linear sum of weights, "weighted" to make this
        a weighted sum, "nearest" to
        use only the weight of the nearest sensor, or "single" to
        do a distance-weight of the nearest sensor. Default is "sum".
    colormap : str
        Colormap to use.
    surface : str
        The type of surface (inflated, white etc.).
    hemi : str
        Hemisphere id (ie 'lh', 'rh', 'both', or 'split'). In the case
        of 'both', both hemispheres are shown in the same window.
        In the case of 'split' hemispheres are displayed side-by-side
        in different viewing panes.
    size : float or tuple of float
        The size of the window, in pixels. can be one number to specify
        a square window, or the (width, height) of a rectangular window.
        Has no effect with mpl backend.
    view : str
        View to set brain to.
    colorbar : bool
        If True, display colorbar on scene.
    distance : float
        Distance (m) defining the activation "ball" of the sensor.
    %(subjects_dir)s
    src : instance of SourceSpaces
        The source space.
    %(verbose)s

    Returns
    -------
    figure : instance of mne.viz.Brain | matplotlib.figure.Figure
        An instance of :class:`mne.viz.Brain` or matplotlib figure.
    """
    info = deepcopy(inst if isinstance(inst, Info) else inst.info)
    if not (info.ch_names == list(statsmodel_df["ch_name"].values)):
        raise RuntimeError(
            "MNE data structure does not match dataframe "
            f"results.\nMNE = {info.ch_names}.\n"
            f"GLM = {list(statsmodel_df['ch_name'].values)}"
        )

    ea = EvokedArray(np.tile(statsmodel_df[value].values.T, (1, 1)).T, info.copy())

    return _plot_3d_evoked_array(
        inst,
        ea,
        picks=picks,
        value=value,
        background=background,
        figure=figure,
        clim=clim,
        mode=mode,
        colormap=colormap,
        surface=surface,
        hemi=hemi,
        size=size,
        view=view,
        colorbar=colorbar,
        distance=distance,
        subjects_dir=subjects_dir,
        src=src,
        verbose=verbose,
    )


def _plot_3d_evoked_array(
    inst,
    ea,
    picks="hbo",
    value="Coef.",
    background="w",
    figure=None,
    clim="auto",
    mode="weighted",
    colormap="RdBu_r",
    surface="pial",
    hemi="both",
    size=800,
    view=None,
    colorbar=True,
    distance=0.03,
    subjects_dir=None,
    src=None,
    verbose=False,
):
    # TODO: mimic behaviour of other MNE-NIRS glm plotting options
    if picks is not None:
        ea = ea.pick(picks=picks)

    if subjects_dir is None:
        subjects_dir = get_subjects_dir(raise_error=True)
    if src is None:
        fname_src_fs = os.path.join(
            subjects_dir, "fsaverage", "bem", "fsaverage-ico-5-src.fif"
        )
        src = read_source_spaces(fname_src_fs)

    picks = np.arange(len(ea.info["ch_names"]))

    # Set coord frame
    for idx in range(len(ea.ch_names)):
        ea.info["chs"][idx]["coord_frame"] = FIFF.FIFFV_COORD_HEAD

    # Generate source estimate
    kwargs = dict(
        evoked=ea,
        subject="fsaverage",
        trans="fsaverage",
        distance=distance,
        mode=mode,
        surface=surface,
        subjects_dir=subjects_dir,
        src=src,
        project=True,
    )
    stc = stc_near_sensors(picks=picks, **kwargs, verbose=verbose)

    # Produce brain plot
    brain = stc.plot(
        src=src,
        subjects_dir=subjects_dir,
        hemi=hemi,
        surface=surface,
        initial_time=0,
        clim=clim,
        size=size,
        colormap=colormap,
        figure=figure,
        background=background,
        colorbar=colorbar,
        verbose=verbose,
    )
    if view is not None:
        brain.show_view(view)

    return brain



# Authors: Robert Luke <mail@robertluke.net>
#
# License: BSD (3-clause)

import numpy as np
import pytest

from mne_nirs.experimental_design import make_first_level_design_matrix
from mne_nirs.simulation import simulate_nirs_raw


def test_simulate_NIRS_single_channel():
    np.random.seed(0)  # TODO: Add random_state and refactor
    raw = simulate_nirs_raw(
        sfreq=3.0,
        amplitude=1.0,
        sig_dur=300.0,
        stim_dur=5.0,
        isi_min=15.0,
        isi_max=45.0,
    )
    assert "hbo" in raw
    assert raw.info["sfreq"] == 3.0
    data = raw.get_data()
    assert data.shape == (1, 900)
    max_val = np.max(data)
    assert max_val < 1.4e-6
    assert raw.annotations.description[0] == "A"
    assert raw.annotations.duration[0] == 5
    assert np.min(np.diff(raw.annotations.onset)) > 15.0 + 5.0
    assert np.max(np.diff(raw.annotations.onset)) < 45.0 + 5.0

    with pytest.raises(AssertionError, match="Same number of"):
        _ = simulate_nirs_raw(
            sfreq=3.0,
            amplitude=[1.0, 2.0],
            sig_dur=300.0,
            stim_dur=5.0,
            isi_min=15.0,
            isi_max=45.0,
        )


def test_simulate_NIRS_multi_channel():
    np.random.seed(0)  # TODO: Add random_state and refactor
    raw = simulate_nirs_raw(
        sfreq=3.0,
        amplitude=[0.0, 2.0, 4.0],
        annot_desc=["Control", "Cond_A", "Cond_B"],
        stim_dur=[5, 5, 5],
        sig_dur=1500.0,
        isi_min=5.0,
        isi_max=15.0,
        hrf_model="spm",
    )

    design_matrix = make_first_level_design_matrix(
        raw, stim_dur=5.0, drift_order=0, drift_model="polynomial"
    )

    assert len(design_matrix["Control"]) == 1500 * 3
    assert len(design_matrix["Cond_A"]) == 1500 * 3

    # Make sure no extra channels. Specifically the default isn't present.
    with pytest.raises(KeyError, match="A"):
        len(design_matrix["A"])



# Authors: Robert Luke  <mail@robertluke.net>
#          simplified BSD-3 license

import datetime
import os.path as op

import h5py
import pandas as pd
import pytest
from mne.datasets.testing import data_path, requires_testing_data
from mne.io import read_raw_nirx, read_raw_snirf
from mne.preprocessing.nirs import beer_lambert_law, optical_density
from mne.utils import check_version, object_diff
from numpy.testing import assert_allclose, assert_array_equal
from snirf import Snirf, validateSnirf

import mne_nirs.datasets.snirf_with_aux as aux
from mne_nirs.io.snirf import SPEC_FORMAT_VERSION, read_snirf_aux_data, write_raw_snirf

fname_nirx_15_0 = op.join(
    data_path(download=False), "NIRx", "nirscout", "nirx_15_0_recording"
)
fname_nirx_15_2 = op.join(
    data_path(download=False), "NIRx", "nirscout", "nirx_15_2_recording"
)
fname_nirx_15_2_short = op.join(
    data_path(download=False), "NIRx", "nirscout", "nirx_15_2_recording_w_short"
)
fname_snirf_aux = aux.data_path()

pytest.importorskip("h5py")

MNE_1_7 = check_version("mne", "1.7")


@requires_testing_data
@pytest.mark.parametrize(
    "fname", (fname_nirx_15_2_short, fname_nirx_15_2, fname_nirx_15_0)
)
def test_snirf_write_raw(fname, tmpdir):
    """Test reading NIRX files."""
    raw_orig = read_raw_nirx(fname, preload=True)
    subj_info = raw_orig.info["subject_info"]
    his_id = "_".join(  # this is how MNE-Python creates his_id for NIRX files
        subj_info[key]
        for key in ("first_name", "middle_name", "last_name")
        if key in subj_info
    )
    assert raw_orig.info["subject_info"]["his_id"] == his_id
    test_file = tmpdir.join("test_raw.snirf")
    write_raw_snirf(raw_orig, test_file)
    raw = read_raw_snirf(test_file, preload=True)
    # Correct MNE bug with reading
    subj_info = raw.info["subject_info"]
    if not MNE_1_7:
        subj_info["first_name"] = subj_info["first_name"].split("_")[0]
        assert "his_id" not in subj_info
        subj_info["his_id"] = his_id
    assert subj_info["his_id"] == his_id

    result = validateSnirf(str(test_file))
    if result.is_valid():
        result.display()
    assert result.is_valid()

    # Check annotations are the same
    assert_allclose(raw.annotations.onset, raw_orig.annotations.onset)
    assert_allclose(
        [float(d) for d in raw.annotations.description],
        [float(d) for d in raw_orig.annotations.description],
    )
    assert_allclose(raw.annotations.duration, raw_orig.annotations.duration)

    # Check data is the same
    assert_allclose(raw.get_data(), raw_orig.get_data())
    assert_array_equal(raw.info.get_channel_types(), raw_orig.info.get_channel_types())

    assert abs(raw_orig.info["meas_date"] - raw.info["meas_date"]) < datetime.timedelta(
        seconds=1
    )

    # Check info object is the same
    obj_diff = object_diff(raw.info, raw_orig.info)
    diffs = ""
    for line in obj_diff.splitlines():
        if (
            ("logno" not in line)
            and ("scanno" not in line)
            and ("his_id" not in line)
            and ("dig" not in line)
            and ("datetime mismatch" not in line)
        ):
            # logno and scanno are not used in processing
            diffs += f"\n{line}"
    assert diffs == ""

    _verify_snirf_required_fields(test_file)
    _verify_snirf_version_str(test_file)

    # make sure one can be written and read with no his_id (e.g., old MNE-Python)
    del raw.info["subject_info"]["his_id"]
    write_raw_snirf(raw, test_file)
    raw = read_raw_snirf(test_file)
    if MNE_1_7:
        assert raw.info["subject_info"]["his_id"] == his_id
    else:
        assert raw.info["subject_info"]["first_name"] == his_id


@requires_testing_data
@pytest.mark.parametrize(
    "fname", (fname_nirx_15_2_short, fname_nirx_15_2, fname_nirx_15_0)
)
def test_snirf_write_optical_density(fname, tmpdir):
    """Test writing optical density SNIRF files."""
    raw_nirx = read_raw_nirx(fname, preload=True)
    od_orig = optical_density(raw_nirx)
    test_file = tmpdir.join("test_od.snirf")
    write_raw_snirf(od_orig, test_file)
    od = read_raw_snirf(test_file)
    assert "fnirs_od" in od

    result = validateSnirf(str(test_file))
    if result.is_valid():
        result.display()
    assert result.is_valid()


@requires_testing_data
@pytest.mark.parametrize(
    "fname",
    (
        fname_nirx_15_2,
        fname_nirx_15_2_short,
    ),
)
def test_snirf_write_haemoglobin(fname, tmpdir):
    """Test haemoglobin writing and reading."""
    raw_nirx = read_raw_nirx(fname, preload=True)
    od_orig = optical_density(raw_nirx)
    hb_orig = beer_lambert_law(od_orig)
    assert hb_orig.annotations.duration[0] == 1
    test_file = tmpdir.join("test_raw_hb_no_mod.snirf")
    write_raw_snirf(hb_orig, test_file)

    result = validateSnirf(str(test_file))
    if result.is_valid():
        result.display()
    assert result.is_valid()

    # HBO

    test_file = tmpdir.join("test_raw_hbo_no_mod.snirf")
    write_raw_snirf(hb_orig.copy().pick("hbo"), test_file)

    result = validateSnirf(str(test_file))
    if result.is_valid():
        result.display()
    assert result.is_valid()

    # HBR

    test_file = tmpdir.join("test_raw_hbr_no_mod.snirf")
    write_raw_snirf(hb_orig.copy().pick("hbr"), test_file)

    result = validateSnirf(str(test_file))
    if result.is_valid():
        result.display()
    assert result.is_valid()


@requires_testing_data
@pytest.mark.parametrize("fname", (fname_nirx_15_2,))
def test_snirf_nobday(fname, tmpdir):
    """Ensure writing works when no birthday is present."""
    raw_orig = read_raw_nirx(fname, preload=True)
    raw_orig.info["subject_info"].pop("birthday", None)
    test_file = tmpdir.join("test_raw.snirf")
    write_raw_snirf(raw_orig, test_file)
    raw = read_raw_snirf(test_file)
    assert_allclose(raw.get_data(), raw_orig.get_data())


@requires_testing_data
@pytest.mark.parametrize("fname", (fname_nirx_15_2,))
def test_snirf_extra_atlasviewer(fname, tmpdir):
    """Ensure writing atlasviewer landmarks."""
    raw_orig = read_raw_nirx(fname, preload=True)
    test_file = tmpdir.join("test_raw.snirf")

    write_raw_snirf(raw_orig, test_file, add_montage=False)
    raw = read_raw_snirf(test_file)
    assert len([i["ident"] for i in raw.info["dig"]]) == 35

    write_raw_snirf(raw_orig, test_file, add_montage=True)
    raw = read_raw_snirf(test_file)
    assert len([i["ident"] for i in raw.info["dig"]]) == 129
    snirf = Snirf(str(test_file), "r")
    assert len(snirf.nirs[0].probe.landmarkLabels) == 129
    assert "Fpz" in snirf.nirs[0].probe.landmarkLabels
    assert "P7" in snirf.nirs[0].probe.landmarkLabels
    assert "HP_20" in snirf.nirs[0].probe.landmarkLabels


def _verify_snirf_required_fields(test_file):
    """Test that all required fields are present.

    Uses version 1.1 of the spec:
    https://raw.githubusercontent.com/fNIRS/snirf/v1.1/snirf_specification.md
    """
    required_metadata_fields = [
        "SubjectID",
        "MeasurementDate",
        "MeasurementTime",
        "LengthUnit",
        "TimeUnit",
        "FrequencyUnit",
    ]
    required_measurement_list_fields = [
        "sourceIndex",
        "detectorIndex",
        "wavelengthIndex",
        "dataType",
        "dataTypeIndex",
    ]

    with h5py.File(test_file, "r") as h5:
        # Verify required base fields
        assert "nirs" in h5
        assert "formatVersion" in h5

        # Verify required metadata fields
        assert "metaDataTags" in h5["/nirs"]
        metadata = h5["/nirs/metaDataTags"]
        for field in required_metadata_fields:
            assert field in metadata

        # Verify required data fields
        assert "data1" in h5["/nirs"]
        data1 = h5["/nirs/data1"]
        assert "dataTimeSeries" in data1
        assert "time" in data1

        # Verify required fields for each measurementList
        measurement_lists = [k for k in data1.keys() if k.startswith("measurementList")]
        for ml in measurement_lists:
            for field in required_measurement_list_fields:
                assert field in data1[ml]

        # Verify required fields for each stimulus
        stims = [k for k in h5["/nirs"].keys() if k.startswith("stim")]
        for stim in stims:
            assert "name" in h5["/nirs"][stim]
            assert "data" in h5["/nirs"][stim]

        # Verify probe fields
        assert "probe" in h5["/nirs"]
        probe = h5["/nirs/probe"]
        assert "wavelengths" in probe
        assert "sourcePos3D" in probe or "sourcePos2D" in probe
        assert "detectorPos3D" in probe or "detectorPos2D" in probe


def _verify_snirf_version_str(test_file):
    """Verify that the version string contains the correct spec version."""
    with h5py.File(test_file, "r") as h5:
        version_str = h5["/formatVersion"][()].decode("UTF-8")
        expected_str = SPEC_FORMAT_VERSION
        assert version_str == expected_str


def test_aux_read():
    """Test reading auxiliary data from SNIRF file."""
    raw = read_raw_snirf(fname_snirf_aux)
    a = read_snirf_aux_data(fname_snirf_aux, raw)
    assert type(a) is pd.DataFrame
    assert "accelerometer_2_z" in a
    assert len(a["gyroscope_1_z"]) == len(raw.times)


@requires_testing_data
@pytest.mark.parametrize(
    "fname",
    (
        fname_nirx_15_2,
        fname_nirx_15_2_short,
    ),
)
def test_snirf_stim_roundtrip(fname, tmpdir):
    """Ensure snirf annotations are written."""
    raw_orig = read_raw_nirx(fname, preload=True)
    assert raw_orig.annotations.duration[0] == 1
    raw_mod = raw_orig.copy()
    test_file = tmpdir.join("test_raw_no_mod.snirf")
    write_raw_snirf(raw_mod, test_file)
    raw = read_raw_snirf(test_file)
    assert_array_equal(raw_orig.annotations.onset, raw.annotations.onset)
    assert_array_equal(raw_orig.annotations.duration, raw.annotations.duration)
    assert_array_equal(raw_orig.annotations.description, raw.annotations.description)


@requires_testing_data
@pytest.mark.parametrize(
    "fname",
    (
        fname_nirx_15_2,
        fname_nirx_15_2_short,
    ),
)
@pytest.mark.parametrize("newduration", (1, 2, 3))
def test_snirf_duration(fname, newduration, tmpdir):
    """Ensure snirf annotations are written to file."""
    pytest.importorskip("mne", "1.4")
    raw_orig = read_raw_nirx(fname, preload=True)
    assert raw_orig.annotations.duration[0] == 1
    raw_mod = raw_orig.copy()
    raw_mod.annotations.set_durations(newduration)
    test_file = tmpdir.join("test_raw_duration.snirf")
    write_raw_snirf(raw_mod, test_file)
    raw = read_raw_snirf(test_file)
    assert raw.annotations.duration[0] == newduration
    assert raw.annotations.duration[-1] == newduration


@requires_testing_data
@pytest.mark.parametrize(
    "fname",
    (
        fname_nirx_15_2,
        fname_nirx_15_2_short,
    ),
)
def test_optical_density_roundtrip(fname, tmpdir):
    """Test optical density writing and reading."""
    raw_nirx = read_raw_nirx(fname, preload=True)
    od_orig = optical_density(raw_nirx)
    assert od_orig.annotations.duration[0] == 1
    test_file = tmpdir.join("test_raw_no_mod.snirf")
    write_raw_snirf(od_orig, test_file)
    od = read_raw_snirf(test_file)
    assert "fnirs_od" in od
    assert_array_equal(od_orig.annotations.onset, od.annotations.onset)
    assert_array_equal(od_orig.annotations.duration, od.annotations.duration)
    assert_array_equal(od_orig.annotations.description, od.annotations.description)
    assert_array_equal(od_orig.get_data(), od.get_data())
    assert_array_equal(od_orig.info.get_channel_types(), od.info.get_channel_types())


@requires_testing_data
@pytest.mark.parametrize(
    "fname",
    (
        fname_nirx_15_2,
        fname_nirx_15_2_short,
    ),
)
def test_haemoglobin_roundtrip(fname, tmpdir):
    """Test haemoglobin writing and reading."""
    raw_nirx = read_raw_nirx(fname, preload=True)
    od_orig = optical_density(raw_nirx)
    hb_orig = beer_lambert_law(od_orig)
    assert hb_orig.annotations.duration[0] == 1
    test_file = tmpdir.join("test_raw_hb_no_mod.snirf")
    write_raw_snirf(hb_orig, test_file)
    hb = read_raw_snirf(test_file)
    assert "hbo" in hb
    assert "hbr" in hb
    assert_array_equal(hb_orig.annotations.onset, hb.annotations.onset)
    assert_array_equal(hb_orig.annotations.duration, hb.annotations.duration)
    assert_array_equal(hb_orig.annotations.description, hb.annotations.description)
    assert_array_equal(hb_orig.get_data(), hb.get_data())
    assert_array_equal(hb_orig.info.get_channel_types(), hb.info.get_channel_types())
    #
    # # HBO
    #
    # hbo = hb_orig.copy().pick("hbo")
    # test_file = tmpdir.join('test_raw_hbo_no_mod.snirf')
    # write_raw_snirf(hbo, test_file)
    # hb = read_raw_snirf(test_file)
    # assert 'hbo' in hb
    # assert 'hbr' not in hb
    # assert_array_equal(hbo.get_data(), hb.get_data())
    # assert_array_equal(hbo.info.get_channel_types(),
    #                    hb.info.get_channel_types())
    #
    # # HBR
    #
    # hbr = hb_orig.copy().pick("hbr")
    # test_file = tmpdir.join('test_raw_hbr_no_mod.snirf')
    # write_raw_snirf(hbr, test_file)
    # hb = read_raw_snirf(test_file)
    # assert 'hbr' in hb
    # assert 'hbo' not in hb
    # assert_array_equal(hbr.get_data(), hb.get_data())
    # assert_array_equal(hbr.info.get_channel_types(),
    #                    hb.info.get_channel_types())



# Authors: Robert Luke <mail@robertluke.net>
#
# License: BSD (3-clause)

# This script runs each of the example scripts. It acts as a system test.

import os
import sys

import pytest
from mne.utils import check_version


def examples_path():
    if not os.path.isdir("BIDS-NIRS-Tapping"):
        os.system(
            "git clone --depth 1 https://github.com/rob-luke/BIDS-NIRS-Tapping.git"
        )

    if os.path.isdir("examples"):
        path = "examples/general/"
    else:
        path = "../examples/general/"

    return path


requires_mne_1p2 = pytest.mark.skipif(
    not check_version("mne", "1.2"), reason="Needs MNE-Python 1.2"
)
# https://github.com/mne-tools/mne-bids/pull/406
try:
    from mne_bids.config import EPHY_ALLOWED_DATATYPES
except Exception:
    missing_mne_bids_fnirs = "Could not import EPHY_ALLOWED_DATATYPES"
else:
    if "nirs" in EPHY_ALLOWED_DATATYPES:
        missing_mne_bids_fnirs = None
    else:
        missing_mne_bids_fnirs = '"nirs" not in EPHY_ALLOWED_DATATYPES'
requires_mne_bids_nirs = pytest.mark.skipif(
    missing_mne_bids_fnirs is not None,
    reason=f"Incorrect MNE-BIDS version: {missing_mne_bids_fnirs}",
)


# TODO: Change ax= to axes= once nilearn 0.11 is out
@pytest.mark.filterwarnings("ignore:No bad channels to interpolate.*:")
@pytest.mark.filterwarnings("ignore:divide by zero encountered.*:")
@pytest.mark.filterwarnings("ignore:invalid value encountered.*:")
@pytest.mark.skipif(sys.platform.startswith("win"), reason="Unstable on Windows")
@pytest.mark.examples
@pytest.mark.parametrize(
    "fname",
    (
        [
            "plot_01_data_io.py",
            pytest.param("plot_05_datasets.py", marks=requires_mne_bids_nirs),
            "plot_10_hrf_simulation.py",
            pytest.param("plot_11_hrf_measured.py", marks=requires_mne_1p2),
            pytest.param("plot_12_group_glm.py", marks=requires_mne_1p2),
            pytest.param("plot_13_fir_glm.py", marks=requires_mne_bids_nirs),
            pytest.param("plot_14_glm_components.py", marks=requires_mne_1p2),
            "plot_15_waveform.py",
            "plot_16_waveform_group.py",
            pytest.param("plot_19_snirf.py", marks=requires_mne_bids_nirs),
            "plot_20_enhance.py",
            "plot_21_artifacts.py",
            "plot_22_quality.py",
            "plot_30_frequency.py",
            "plot_40_mayer.py",
            pytest.param("plot_80_save_read_glm.py", marks=requires_mne_bids_nirs),
            "plot_99_bad.py",
        ]
    ),
)
def test_examples(fname, requires_pyvista):
    test_file_path = examples_path() + fname
    with open(test_file_path) as fid:
        code = fid.read()
    exec(code, locals(), locals())



"""Download useful datasets."""

# Authors: Robert Luke <mail@robertluke.net>
#
# License: BSD (3-clause)

from . import audio_or_visual_speech
from . import camh_kf_fnirs_fingertapping
from . import block_speech_noise
from . import fnirs_motor_group
from . import snirf_with_aux



# Authors: Robert Luke <mail@robertluke.net>
#
# License: BSD (3-clause)

import nilearn
import numpy as np
import pytest
from mne import Covariance
from mne.simulation import add_noise

from mne_nirs.experimental_design import make_first_level_design_matrix
from mne_nirs.simulation import simulate_nirs_raw
from mne_nirs.statistics import run_GLM, run_glm

iir_filter = [1.0, -0.58853134, -0.29575669, -0.52246482, 0.38735476, 0.024286]


def test_run_GLM():
    raw = simulate_nirs_raw(sig_dur=200, stim_dur=5.0)
    design_matrix = make_first_level_design_matrix(
        raw, stim_dur=5.0, drift_order=1, drift_model="polynomial"
    )
    glm_estimates = run_glm(raw, design_matrix)

    # Test backwards compatibility
    with pytest.deprecated_call(match="more comprehensive"):
        old_res = run_GLM(raw, design_matrix)
    assert old_res.keys() == glm_estimates.data.keys()
    assert (old_res["Simulated"].theta == glm_estimates.data["Simulated"].theta).all()

    assert len(glm_estimates) == len(raw.ch_names)

    # Check the estimate is correct within 10% error
    assert abs(glm_estimates.pick("Simulated").theta()[0][0] - 1.0e-6) < 0.1e-6

    # ensure we return the same type as nilearn to encourage compatibility
    _, ni_est = nilearn.glm.first_level.run_glm(raw.get_data(0).T, design_matrix.values)
    assert isinstance(glm_estimates._data, type(ni_est))


def test_run_GLM_order():
    raw = simulate_nirs_raw(sig_dur=200, stim_dur=5.0, sfreq=3)
    design_matrix = make_first_level_design_matrix(
        raw, stim_dur=5.0, drift_order=1, drift_model="polynomial"
    )

    # Default should be first order AR
    glm_estimates = run_glm(raw, design_matrix)
    assert glm_estimates.pick("Simulated").model()[0].order == 1

    # Default should be first order AR
    glm_estimates = run_glm(raw, design_matrix, noise_model="ar2")
    assert glm_estimates.pick("Simulated").model()[0].order == 2

    glm_estimates = run_glm(raw, design_matrix, noise_model="ar7")
    assert glm_estimates.pick("Simulated").model()[0].order == 7

    # Auto should be 4 times sample rate
    cov = Covariance(
        np.ones(1) * 1e-11, raw.ch_names, raw.info["bads"], raw.info["projs"], nfree=0
    )
    raw = add_noise(raw, cov, iir_filter=iir_filter)
    glm_estimates = run_glm(raw, design_matrix, noise_model="auto")
    assert glm_estimates.pick("Simulated").model()[0].order == 3 * 4

    raw = simulate_nirs_raw(sig_dur=10, stim_dur=5.0, sfreq=2)
    cov = Covariance(
        np.ones(1) * 1e-11, raw.ch_names, raw.info["bads"], raw.info["projs"], nfree=0
    )
    raw = add_noise(raw, cov, iir_filter=iir_filter)
    design_matrix = make_first_level_design_matrix(
        raw, stim_dur=5.0, drift_order=1, drift_model="polynomial"
    )
    # Auto should be 4 times sample rate
    glm_estimates = run_glm(raw, design_matrix, noise_model="auto")
    assert glm_estimates.pick("Simulated").model()[0].order == 2 * 4
